import typing

import colorama

import random

colorama.init()

__all__: typing.List[str] = \
    [
        "color_it_full",
        "color_it216",
        "color_it48",
        "DIZERING_RANGE"
    ]

DIZERING_RANGE: int = 3

pal: typing.List[int] = [0, 95, 135, 175, 215, 255]

colors: typing.Dict[str, typing.List[str]] = \
    {"GREEN": [colorama.Style.DIM + colorama.Fore.GREEN,
               colorama.Fore.GREEN,
               colorama.Style.BRIGHT + colorama.Fore.GREEN,
               colorama.Style.DIM + colorama.Fore.LIGHTGREEN_EX,
               colorama.Fore.LIGHTGREEN_EX, colorama.Style.BRIGHT +
               colorama.Fore.LIGHTGREEN_EX
               ],
     "RED": [colorama.Style.DIM + colorama.Fore.RED,
             colorama.Fore.RED,
             colorama.Style.BRIGHT + colorama.Fore.RED,
             colorama.Style.DIM + colorama.Fore.LIGHTRED_EX,
             colorama.Fore.LIGHTRED_EX,
             colorama.Style.BRIGHT + colorama.Fore.LIGHTRED_EX
             ],
     "YELLOW": [colorama.Style.DIM + colorama.Fore.YELLOW,
                colorama.Fore.YELLOW,
                colorama.Style.BRIGHT + colorama.Fore.YELLOW,
                colorama.Style.DIM + colorama.Fore.LIGHTYELLOW_EX,
                colorama.Fore.LIGHTYELLOW_EX, colorama.Style.BRIGHT +
                colorama.Fore.LIGHTYELLOW_EX
                ],
     "MAGENTA": [colorama.Style.DIM + colorama.Fore.MAGENTA,
                 colorama.Fore.MAGENTA,
                 colorama.Style.BRIGHT + colorama.Fore.MAGENTA,
                 colorama.Style.DIM + colorama.Fore.LIGHTMAGENTA_EX,
                 colorama.Fore.LIGHTMAGENTA_EX, colorama.Style.BRIGHT +
                 colorama.Fore.LIGHTMAGENTA_EX
                 ],
     "CYAN": [colorama.Style.DIM + colorama.Fore.CYAN,
              colorama.Fore.CYAN,
              colorama.Style.BRIGHT + colorama.Fore.CYAN,
              colorama.Style.DIM + colorama.Fore.LIGHTCYAN_EX,
              colorama.Fore.LIGHTCYAN_EX,
              colorama.Style.BRIGHT + colorama.Fore.LIGHTCYAN_EX
              ],
     "WHITE": [colorama.Style.DIM + colorama.Fore.WHITE,
               colorama.Fore.WHITE,
               colorama.Style.BRIGHT + colorama.Fore.WHITE,
               colorama.Style.DIM + colorama.Fore.LIGHTWHITE_EX,
               colorama.Fore.LIGHTWHITE_EX,
               colorama.Style.BRIGHT + colorama.Fore.LIGHTWHITE_EX
               ],
     "BLACK": [colorama.Style.DIM + colorama.Fore.BLACK,
               colorama.Fore.BLACK,
               colorama.Style.BRIGHT + colorama.Fore.BLACK,
               colorama.Style.DIM + colorama.Fore.LIGHTBLACK_EX,
               colorama.Fore.LIGHTBLACK_EX, colorama.Style.BRIGHT +
               colorama.Fore.LIGHTBLACK_EX
               ],
     "BLUE": [colorama.Style.DIM + colorama.Fore.BLUE,
              colorama.Fore.BLUE,
              colorama.Style.BRIGHT + colorama.Fore.BLUE,
              colorama.Style.DIM + colorama.Fore.LIGHTBLUE_EX,
              colorama.Fore.LIGHTBLUE_EX,
              colorama.Style.BRIGHT + colorama.Fore.LIGHTBLUE_EX
              ]
     }


def get_pal(color: typing.Tuple[int, int, int]) -> typing.List[int]:
    """Get nearest value of pal to color's rgb"""
    col_data: typing.List[int] = []
    for col in color:
        added: bool = False
        for i in enumerate(pal[1:]):
            added = False
            if (col - pal[i[0]]) / (i[1] - pal[i[0]]) < 0.5:
                col_data.append(i[0])
                added = True
                break
        if not added:
            col_data.append(len(pal) - 1)
    return col_data


def color_it48(color: typing.Tuple[int, int, int],
               dizering: bool = False) -> str:
    """48 colors"""
    if all([col > 240 for col in color]) \
            and color[0] * 3 - 10 < sum(color) < color[0] * 3 + 10:
        return colors["WHITE"][
            round((len(colors["WHITE"]) - 1)
                  / 255 * (color[1] + color[2]) / 2)]
    if all([col < 30 for col in color]) \
            and color[0] * 3 - 10 < sum(color) < color[0] * 3 + 10:
        return colors["BLACK"][
            round((len(colors["BLACK"]) - 1)
                  / 255 * (color[1] + color[2]) / 2)]
    if max(color) == color[1] and color[1] > color[0] + color[2] - 20:
        return colors["GREEN"][
            round((len(colors["GREEN"]) - 1)
                  / 255 * color[1])]
    if max(color) == color[0] and color[0] > sum(color[1:3]) - 20:
        return colors["RED"][
            round((len(colors["RED"]) - 1)
                  / 255 * color[0])]
    if max(color) == color[2] and color[2] > sum(color[0:2]) - 20:
        return colors["BLUE"][
            round((len(colors["BLUE"]) - 1)
                  / 255 * color[0])]
    if color[1] + color[2] > color[0] * 2 + 40:
        return colors["CYAN"][
            round((len(colors["CYAN"]) - 1)
                  / 255 * (color[1] + color[2]) / 2)]
    if color[0] + color[2] > color[1] * 2 + 40:
        return colors["MAGENTA"][
            round((len(colors["MAGENTA"]) - 1)
                  / 255 * (color[2] + color[1]) / 2)]
    if sum(color[0:2]) > color[2] * 2 + 40:
        return colors["YELLOW"][
            round((len(colors["YELLOW"]) - 1)
                  / 255 * (color[1] + color[0]) / 2)]
    return ""


def color_it216(color: typing.Tuple[int, int, int],
                dizering: bool = False) -> str:
    """216 colors"""
    color_data: typing.List[int] = get_pal(color)
    color_num: int = sum([6 ** (len(color_data) - index - 1) * data
                          for index, data in enumerate(color_data)])
    dizering_number: int = random.randint(0, int(dizering) * DIZERING_RANGE*2)

    return f"\033[38;05;" \
           f"{16 + color_num};48;05" \
           f"{16 + color_num - dizering_number}m"


def color_it_full(color: typing.Tuple[int, int, int],
                  dizering: bool = True) -> str:
    """Full rgb"""
    color_addings: typing.List = [0, 0, 0]
    if dizering:
        color_addings = [random.randint(0,
                                        DIZERING_RANGE * 2) - DIZERING_RANGE,
                         random.randint(0,
                                        DIZERING_RANGE * 2) - DIZERING_RANGE,
                         random.randint(0,
                                        DIZERING_RANGE * 2) - DIZERING_RANGE]
        for index, adding in enumerate(color_addings):
            if color[index] - adding < 0:
                color_addings[index] = 0
    return f"\033[48;02;{color[0] - color_addings[0]};" \
           f"{color[1] - color_addings[1]};" \
           f"{color[2] - color_addings[2]};38;02;" \
           f"{color[0]};{color[1]};{color[2]}m"



from PIL import Image, ImageDraw

"""GIVES YOU BRIGHT TABLE"""

s: dict[str, int] = {}
for i in range(12, 10000):
    image: Image = Image.new("L", (10, 14))
    try:
        ImageDraw.Draw(image).text((0, 0), chr(i), 255)
    except UnicodeError:
        continue
    c = 0
    for x in range(image.height):
        for y in range(image.width):
            if image.getpixel((y, x)) == 255:
                c += 1
    if c not in s.items():
        s[chr(i)] = c
for i in sorted(s.items(), key=lambda n: n[1]):
    print(i[0], end="")



import typing

import colorama

from dataclasses import dataclass
from PIL import Image, ImageSequence
from time import sleep

from .symbol_table import SymbolTable
from .term import get_terminal_size, clear_term
from .palette import color_it_full

colorama.init()

__all__: typing.Sequence[str] = ["generate_image",
                                 "print_image",
                                 "print_images",
                                 "show_gif",
                                 "show_gifs",
                                 "TextImage",
                                 "ascii_table",
                                 "block_table",
                                 "fucking_big_table"]

ascii_table: SymbolTable = SymbolTable([' ', '.', ':', "'",
                                        ',', '-', ';', '_',
                                        '"', '+', '°', '•',
                                        '^', '=', '|', '(',
                                        '[', '{', "∆", '©',
                                        '®', '&', "$", "@",
                                        '%', '#', '¶'])
block_table: SymbolTable = SymbolTable([" ", "▂", "▃", "▄",
                                        "▅", "▆", "▇", "█"])
fucking_big_table: SymbolTable = SymbolTable([char
                                              for
                                              char in
                                              (".¨´·',:`¯¸-;_~°|¬³¹÷/=\\^¦²" +
                                               "*+×!<>ºª()?¡¤±¿vIc{}Jr¢ìíïF" +
                                               "LTV[]eimow»¼½¾ÌÍÏæî7Cjnsz©«" +
                                               "Îç&1WXYltx§®Þèéëòóö%£êôø#24" +
                                               "EGOZyÇÝàáäAKÐßâõ3MPSafuBDNQ" +
                                               "Ubhkµ¶ÒÓÖØû8RpÀÁÄÈÉËÔãñýÿ0d" +
                                               "ÛðÃgq¥ÂÊÙÅþÑÚÜå$HÆÕ")[::-1]])


@dataclass
class TextImage:
    image: Image.Image
    size: typing.Tuple[int, int]
    table: SymbolTable = block_table

    def to_convertable(self) -> Image.Image:
        """Make image ready to text convert"""
        image = self.image.convert("RGB")
        image = image.resize(self.size)

        return image


def print_image(text_image: TextImage,
                color_function: typing.Callable[
                    [typing.Tuple
                     [int, int, int],
                     bool], str] = color_it_full,
                dizering: bool = False
                ) -> None:
    """Print one image"""
    image: Image.Image \
        = text_image.to_convertable()
    image_grayscale: Image.Image = image.convert("L")
    for i in range(0, image.height):
        result: str = ""
        for j in range(0, image.width):
            result += color_function(image.getpixel((j, i)),
                                     dizering) + \
                      text_image.table.get_symbol(
                          image_grayscale.getpixel((j, i)))
        print(result)
    print(colorama.Style.RESET_ALL)


def print_images(*text_images: TextImage,
                 sep: str = "",
                 color_function: typing.Callable[
                     [typing.Tuple
                      [int, int, int],
                      bool], str] = color_it_full,
                 dizering: bool = False
                 ) -> None:
    """Print many images separated by sep"""
    image_generators: typing.Dict[typing.Generator[
                                      str, None, None], TextImage] = \
        {generate_image(t_image[1],
                        color_function=color_function,
                        dizering=dizering): t_image[1]
         for t_image in enumerate(text_images)}
    max_size: int = max([t_image.size[1] for t_image in text_images])

    for current_line in range(max_size):
        print(*[next(img_generator)
                for img_generator, t_image_index
                in image_generators.items()],
              sep=sep)
    print(colorama.Style.RESET_ALL)


def generate_image(text_image: TextImage,
                   color_function: typing.Callable[
                       [typing.Tuple
                        [int, int, int],
                        bool], str] = color_it_full,
                   dizering: bool = False
                   ) -> typing.Generator[str, None, None]:
    """String generator of image"""
    image: Image.Image = text_image.to_convertable()
    image_grayscale: Image.Image = image.convert("L")
    for i in range(0, image.height):
        result: str = ""
        for j in range(0, image.width):
            result += color_function(image.getpixel((j, i)),
                                     dizering) + \
                      text_image.table.get_symbol(
                          image_grayscale.getpixel((j, i)))
        yield result


def show_gif(path: str,
             frame_rate: int = 30,
             table: SymbolTable = block_table,
             color_function: typing.Callable[
                 [typing.Tuple
                  [int, int, int],
                  bool], str] = color_it_full,
             dizering: bool = False
             ) -> None:
    """Slideshow in a console, doesnt work with mp4"""
    gif: Image.Image = Image.open(path)
    while True:
        for image in ImageSequence.Iterator(gif):
            columns, rows = get_terminal_size()
            text_image: TextImage = TextImage(image,
                                              (columns, rows),
                                              table=table)
            image = text_image.to_convertable()
            image_grayscale: Image.Image = image.convert("L")
            for i in range(0, image.height):
                for j in range(0, image.width):
                    print(
                        color_function(
                            image.getpixel((j, i)), dizering) +
                        table.get_symbol(
                            image_grayscale.getpixel((j, i))),
                        end='')
                print()

            sleep(1 / frame_rate)
            clear_term()


def show_gifs(*gif_paths: str,
              size: tuple[int, int],
              sep: str = "",
              frame_rate: int = 30,
              table: SymbolTable = block_table,
              color_function: typing.Callable[
                 [typing.Tuple
                  [int, int, int],
                  bool], str] = color_it_full,
              dizering: bool = False
              ) -> None:
    """Many slideshows!"""
    gifs: list[Image.Image] = [Image.open(path) for path in gif_paths]
    while True:
        gifs_iters: list[ImageSequence.Iterator] = \
            [ImageSequence.Iterator(gif) for gif in gifs]

        for image in gifs_iters[0]:
            clear_term()
            print_images(
                *[TextImage(image, size, table=table)] +
                 [TextImage(next(gif_iter),
                            size,
                            table=table)
                  for gif_iter in gifs_iters[1:]],
                sep=sep,
                color_function=color_function,
                dizering=dizering)
            sleep(1 / frame_rate)



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

requirements = ["colorama", "pillow", "dataclasses"]

setuptools.setup(
    name="console_images",
    version="0.1.1",
    author="LedinecMing",
    url="https://github.com/LedinecMing/console_images/",
    author_email="loliamalexxaxa@gmail.com",
    description="Colored console images and gifs",
    long_description=long_description,
    long_description_content_type="text/markdown",
    packages=setuptools.find_packages(),
    install_requires=requirements,
    classifiers=[
        "Programming Language :: Python :: 3.9",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.8',
)



import typing

from dataclasses import dataclass


@dataclass
class SymbolTable:
    table: typing.MutableSequence[str]

    def get_symbol(self, bright: int):
        """Get symbol of table by bright"""
        return self.table[round(bright / 256 * (len(self.table) - 1))]

    def __getitem__(self, item: int):
        """Alternative for SymbolTable(...).get_symbol(...)"""
        return self.get_symbol(item)



import typing

import os
import platform

import warnings

BASE_SIZE: typing.Tuple[int, int] = (100, 70)


def clear_term() -> None:
    if platform.system() == "Linux":
        os.system("clear")
    elif platform.system() == "Windows":
        os.system("cls")


def get_terminal_size() -> typing.Tuple[int, int]:
    columns: int
    rows: int
    try:
        columns, rows = os.get_terminal_size(0)
    except OSError:
        columns, rows = BASE_SIZE
        warnings.warn("Can not get terminal size, going to use BASE_SIZE")
    return columns, rows



from .to_ascii import *
from .palette import *
from .term import get_terminal_size

if __name__ == "__main__":
    show_gifs("example.gif",
              size=get_terminal_size(),
              color_function=color_it_full,
              dizering=True)
